'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
}

var defaultOptions$1 = {
    maxCache: 10,
};
var TileCache = /** @class */ (function () {
    function TileCache(options) {
        this.items = {};
        this.lru = []; // order by [least recently used, ..., most recently used]
        this.options = __assign(__assign({}, defaultOptions$1), options);
    }
    TileCache.prototype.removeLeastRecentlyUsed = function () {
        var i = 0;
        while (i < this.lru.length && this.lru.length >= this.options.maxCache) {
            var key = this.lru[i];
            if (this.items[key].listeners) {
                i += 1;
                continue;
            }
            delete this.items[key];
            this.lru.splice(i, 1);
        }
    };
    TileCache.prototype.touchItem = function (key) {
        var tileIndex = this.lru.indexOf(key);
        if (tileIndex > -1) {
            this.lru.splice(tileIndex, 1);
            this.lru.push(key);
        }
    };
    TileCache.prototype.pushItem = function (key, item) {
        this.items[key] = item;
        this.lru.push(key);
    };
    TileCache.prototype.publish = function (key, item, error) {
        var _a;
        this.items[key].item = item;
        if (error)
            this.items[key].error = "".concat(error);
        (_a = this.items[key].listeners) === null || _a === void 0 ? void 0 : _a.forEach(function (listener) { return listener(item, error); });
        delete this.items[key].listeners;
        this.removeLeastRecentlyUsed();
    };
    TileCache.prototype.subscribe = function (key, listener) {
        var _this = this;
        var cacheItem = this.items[key];
        if ((cacheItem === null || cacheItem === void 0 ? void 0 : cacheItem.item) !== undefined) {
            // Hit. Item resolved in cache
            this.touchItem(key);
            listener(cacheItem.item, cacheItem.error);
        }
        else if (cacheItem === null || cacheItem === void 0 ? void 0 : cacheItem.listeners) {
            // Hit. Item pending in cache
            this.touchItem(key);
            cacheItem.listeners.push(listener);
        }
        else {
            // Miss. Item not in cache
            this.pushItem(key, {
                listeners: [listener]
            });
            this.options.fetchTile(key)
                .then(function (value) { return _this.publish(key, value); })
                .catch(function (error) { return _this.publish(key, null, error); });
        }
    };
    TileCache.prototype.getTile = function (key) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        _this.subscribe(key, function (item, error) {
                            if (error)
                                return reject(error);
                            resolve(item);
                        });
                    })];
            });
        });
    };
    return TileCache;
}());

var createCanvasContext = function (width, height) {
    var canvas = document.createElement('canvas');
    canvas.width = width;
    canvas.height = height;
    var ctx = canvas.getContext('2d');
    if (!ctx)
        throw new Error('Failed to create canvas context 2d');
    return ctx;
};
var canvasToArrayBuffer = function (canvas) { return __awaiter(void 0, void 0, void 0, function () {
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, new Promise(function (resolve, reject) {
                    canvas.toBlob(function (blob) { return __awaiter(void 0, void 0, void 0, function () {
                        var buf, error_1;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    if (!blob)
                                        return [2 /*return*/, resolve(null)];
                                    _a.label = 1;
                                case 1:
                                    _a.trys.push([1, 3, , 4]);
                                    return [4 /*yield*/, blob.arrayBuffer()];
                                case 2:
                                    buf = _a.sent();
                                    resolve(buf);
                                    return [3 /*break*/, 4];
                                case 3:
                                    error_1 = _a.sent();
                                    reject(error_1);
                                    return [3 /*break*/, 4];
                                case 4: return [2 /*return*/];
                            }
                        });
                    }); });
                })];
            case 1: return [2 /*return*/, _a.sent()];
        }
    });
}); };
var fetchImage = function (url) { return __awaiter(void 0, void 0, void 0, function () {
    return __generator(this, function (_a) {
        return [2 /*return*/, new Promise(function (resolve, reject) { return __awaiter(void 0, void 0, void 0, function () {
                var image, error_2;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            _a.trys.push([0, 2, , 3]);
                            image = new Image();
                            image.crossOrigin = '';
                            image.src = url;
                            return [4 /*yield*/, image.decode()];
                        case 1:
                            _a.sent();
                            resolve(image);
                            return [3 /*break*/, 3];
                        case 2:
                            error_2 = _a.sent();
                            reject(error_2);
                            return [3 /*break*/, 3];
                        case 3: return [2 /*return*/];
                    }
                });
            }); })];
    });
}); };

var drawDestination = function (ctx, source, destination) {
    var destinationTileSize = ctx.destinationTileSize, sourceTileSize = ctx.sourceTileSize, interval = ctx.interval, sourceToPixel = ctx.sourceToPixel, destinationToSource = ctx.destinationToSource, pixelToDestination = ctx.pixelToDestination, destinationToPixel = ctx.destinationToPixel;
    var destinationCanvasCtx = createCanvasContext(destinationTileSize, destinationTileSize);
    var destinationZoom = destination.tile[2];
    var dp0 = destinationToPixel([destination.bbox[0], destination.bbox[1]], destinationZoom, destinationTileSize);
    var dp1 = destinationToPixel([destination.bbox[2], destination.bbox[3]], destinationZoom, destinationTileSize);
    var dTranslate = [
        Math.min(dp0[0], dp1[0]),
        Math.min(dp0[1], dp1[1]),
    ];
    for (var dx = 0; dx < destinationTileSize; dx += interval[0]) {
        // Handle when tileSize is not divisible by x interval
        var dWidth = Math.min(interval[0], destinationTileSize - dx);
        for (var dy = 0; dy < destinationTileSize; dy += interval[1]) {
            // Handle when tileSize is not divisible by y interval
            var dHeight = Math.min(interval[1], destinationTileSize - dy);
            var dBbox = [
                dx + dTranslate[0],
                dy + dTranslate[1],
                dx + dTranslate[0] + dWidth,
                dy + dTranslate[1] + dHeight,
            ];
            var sp0 = sourceToPixel(destinationToSource(pixelToDestination([dBbox[0], dBbox[1]], destinationZoom, destinationTileSize)), source.zoom, sourceTileSize);
            var sp1 = sourceToPixel(destinationToSource(pixelToDestination([dBbox[2], dBbox[3]], destinationZoom, destinationTileSize)), source.zoom, sourceTileSize);
            var sBbox = [
                Math.min(sp0[0], sp1[0]),
                Math.min(sp0[1], sp1[1]),
                Math.max(sp0[0], sp1[0]),
                Math.max(sp0[1], sp1[1]),
            ];
            // Draw slice of destinaton tile
            destinationCanvasCtx.drawImage(source.canvas, sBbox[0] - source.translate[0], // sx 
            sBbox[1] - source.translate[1], // sy
            sBbox[2] - sBbox[0], // sWidth 
            sBbox[3] - sBbox[1], // sHeight, 
            dx, // dx 
            dy, // dy, 
            dWidth, // dWidth, 
            dHeight // dHeight
            );
        }
    }
    return {
        canvas: destinationCanvasCtx.canvas,
        translate: dTranslate,
        zoom: destinationZoom,
    };
};

var drawSource = function (ctx, sources) {
    var zoom = sources[0].tile[2];
    var sp0 = ctx.sourceToPixel([
        Math.min.apply(Math, sources.map(function (d) { return d.bbox[0]; })),
        Math.min.apply(Math, sources.map(function (d) { return d.bbox[1]; }))
    ], zoom, ctx.sourceTileSize);
    var sp1 = ctx.sourceToPixel([
        Math.max.apply(Math, sources.map(function (d) { return d.bbox[2]; })),
        Math.max.apply(Math, sources.map(function (d) { return d.bbox[3]; }))
    ], zoom, ctx.sourceTileSize);
    var sBbox = [
        Math.min(sp0[0], sp1[0]),
        Math.min(sp0[1], sp1[1]),
        Math.max(sp0[0], sp1[0]),
        Math.max(sp0[1], sp1[1]),
    ];
    var canvasCtx = createCanvasContext(sBbox[2] - sBbox[0], // width
    sBbox[3] - sBbox[1] // height
    );
    // Create working canvas to access fetched images
    for (var _i = 0, sources_1 = sources; _i < sources_1.length; _i++) {
        var _a = sources_1[_i], bbox = _a.bbox, image = _a.image;
        if (!image)
            continue;
        var t0 = ctx.sourceToPixel([bbox[0], bbox[1]], zoom, ctx.sourceTileSize);
        var t1 = ctx.sourceToPixel([bbox[2], bbox[3]], zoom, ctx.sourceTileSize);
        var tBbox = [
            Math.min(t0[0], t1[0]),
            Math.min(t0[1], t1[1]),
            Math.max(t0[0], t1[0]),
            Math.max(t0[1], t1[1]),
        ];
        canvasCtx.drawImage(image, tBbox[0] - sBbox[0], // sx
        tBbox[1] - sBbox[1] // sy
        );
    }
    return {
        translate: [sBbox[0], sBbox[1]],
        zoom: zoom,
        canvas: canvasCtx.canvas
    };
};

var drawTile = function (ctx, sources, destinationRequest) {
    // Create new tile image from source tiles
    var source = drawSource(ctx, sources);
    // Use source canvs to render destination canvas
    return drawDestination(ctx, source, destinationRequest);
};

var loadTile = function (_a) {
    var ctx = _a.ctx, destinationRequest = _a.destinationRequest, sourceRequests = _a.sourceRequests, checkCanceled = _a.checkCanceled;
    return __awaiter(void 0, void 0, void 0, function () {
        var sources;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    // Bail when already canceled 
                    if (checkCanceled())
                        return [2 /*return*/, null];
                    return [4 /*yield*/, Promise.all(sourceRequests.map(function (source) { return __awaiter(void 0, void 0, void 0, function () {
                            var image;
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, ctx.cache.getTile(source.url)
                                            .catch(function () { return null; })];
                                    case 1:
                                        image = _a.sent();
                                        return [2 /*return*/, __assign(__assign({}, source), { image: image })];
                                }
                            });
                        }); }))];
                case 1:
                    sources = _b.sent();
                    // Return all sources with an image
                    sources = sources.filter(function (source) { return source.image; });
                    // Validate sources before continue
                    // - Bail when no remaining sources
                    // - Bail when canceled after async source tile fetching
                    if (!(sources === null || sources === void 0 ? void 0 : sources.length) || checkCanceled())
                        return [2 /*return*/, null];
                    // Reproject source tiles to destination tile
                    return [2 /*return*/, drawTile(ctx, sources, destinationRequest)];
            }
        });
    });
};

var parseUrl = function (url) {
    var _a, _b, _c, _d;
    var _e = url.split(/:\/\//), reprojParamsStr = _e[1], urlTemplates = _e.slice(2);
    var urlTemplate = urlTemplates.join('://');
    var reprojParams = new URLSearchParams(reprojParamsStr);
    var tile = [
        +((_a = reprojParams.get('x')) !== null && _a !== void 0 ? _a : 0),
        +((_b = reprojParams.get('y')) !== null && _b !== void 0 ? _b : 0),
        +((_c = reprojParams.get('z')) !== null && _c !== void 0 ? _c : 0),
    ];
    var bbox = ((_d = reprojParams.get('bbox')) !== null && _d !== void 0 ? _d : '').split(',').map(function (d) { return +d; });
    return { bbox: bbox, tile: tile, urlTemplate: urlTemplate };
};
var getImageUrl = function (urlTemplate, sourceTile, sourceBbox) {
    return (urlTemplate
        .replace('{bbox-source}', "".concat(sourceBbox.join(',')))
        .replace('{xmin-source}', "".concat(sourceBbox[0]))
        .replace('{ymin-source}', "".concat(sourceBbox[1]))
        .replace('{xmax-source}', "".concat(sourceBbox[2]))
        .replace('{ymax-source}', "".concat(sourceBbox[3]))
        .replace('{x-source}', "".concat(sourceTile[0]))
        .replace('{y-source}', "".concat(sourceTile[1]))
        .replace('{z-source}', "".concat(sourceTile[2])));
};

var MTA_PROTOCOL = 'mta';
var loader = function (ctx, reqParams, cb) {
    var request = parseUrl(reqParams.url);
    var sourceRequests = ctx.destinationTileToSourceTiles({
        bbox: request.bbox,
        tile: request.tile,
    }).map(function (d) { return (__assign(__assign({}, d), { url: getImageUrl(request.urlTemplate, d.tile, d.bbox) })); });
    var isCanceled = false;
    void loadTile({
        ctx: ctx,
        sourceRequests: sourceRequests,
        destinationRequest: { tile: request.tile, bbox: request.bbox },
        checkCanceled: function () { return isCanceled; },
    })
        .then(function (destination) { return __awaiter(void 0, void 0, void 0, function () {
        var img;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!destination)
                        return [2 /*return*/, cb(null, null)];
                    return [4 /*yield*/, canvasToArrayBuffer(destination.canvas)];
                case 1:
                    img = _a.sent();
                    cb(null, img);
                    return [2 /*return*/];
            }
        });
    }); })
        .catch(function (error) { return cb(error); });
    return {
        cancel: function () {
            isCanceled = true;
        }
    };
};
var maplibreTileAdapterProtocol = function (options) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    var cache = new TileCache({
        fetchTile: function (url) { return fetchImage(url); },
        maxCache: (_a = options.cacheSize) !== null && _a !== void 0 ? _a : 10,
    });
    var destinationTileSize = (_c = (_b = options === null || options === void 0 ? void 0 : options.destinationTileSize) !== null && _b !== void 0 ? _b : options === null || options === void 0 ? void 0 : options.sourceTileSize) !== null && _c !== void 0 ? _c : 256;
    var ctx = {
        cache: cache,
        destinationTileSize: (_e = (_d = options === null || options === void 0 ? void 0 : options.destinationTileSize) !== null && _d !== void 0 ? _d : options === null || options === void 0 ? void 0 : options.sourceTileSize) !== null && _e !== void 0 ? _e : 256,
        destinationTileToSourceTiles: options.destinationTileToSourceTiles,
        destinationToPixel: options.destinationToPixel,
        destinationToSource: options.destinationToSource,
        interval: (_f = options === null || options === void 0 ? void 0 : options.interval) !== null && _f !== void 0 ? _f : [destinationTileSize, destinationTileSize],
        pixelToDestination: options.pixelToDestination,
        sourceTileSize: (_h = (_g = options === null || options === void 0 ? void 0 : options.sourceTileSize) !== null && _g !== void 0 ? _g : options === null || options === void 0 ? void 0 : options.destinationTileSize) !== null && _h !== void 0 ? _h : 256,
        sourceToPixel: options.sourceToPixel,
    };
    return {
        protocol: "".concat((_j = options === null || options === void 0 ? void 0 : options.protocol) !== null && _j !== void 0 ? _j : MTA_PROTOCOL),
        tileUrlPrefix: "".concat((_k = options === null || options === void 0 ? void 0 : options.protocol) !== null && _k !== void 0 ? _k : MTA_PROTOCOL, "://bbox={bbox-epsg-3857}&z={z}&x={x}&y={y}"),
        loader: loader.bind(null, ctx),
    };
};

/**
 * Convert longitude so that -180 is 0 and 180 is 1
 *
 * @param lng longitude in decimal degrees
 * @returns value between 0 and 1
 */
var normalizeLng = function (lng) { return lng / 360 + 0.5; };
/**
 * Convert latitude so that 90 is 0 and -90 is 1
 *
 * @param lat latitude in decimal degrees
 * @returns value between 0 and 1
 */
var normalizeLat = function (lat) { return -lat / 180 + 0.5; };
/**
 * Get the total x (columns) and y (rows) tiles at a zoom level
 *
 * @example
 * ```
 * getExtent(0); // [2, 1]
 * getExtent(1); // [4, 2]
 * getExtent(2); // [8, 4]
 * ```
 *
 * @param z zoom
 * @returns amount of tiles
 */
var getExtent = function (z) {
    var fz = Math.floor(z);
    // same as [2**(z+1), 2**(z)] but slightly faster
    return [2 << fz, 1 << fz];
};
/**
 * Get the precise fractional tile location for a point at a zoom level
 *
 * @param lng longitude in decimal degrees
 * @param lat latitude in decimal degrees
 * @param z zoom
 * @returns tile [x, y, z]
 */
var pointToTileFraction = function (lng, lat, z) {
    // Calculate total tiles at the zoom level
    var _a = getExtent(z), matrixWidth = _a[0], matrixHeight = _a[1];
    // Translate lngLat from merdian to top left corner
    // Then normalize/scale by total degrees to get 0 to 1 based results
    var dx = normalizeLng(lng);
    var dy = normalizeLat(lat);
    // Find fractional tile grid
    var x = dx * matrixWidth;
    var y = dy * matrixHeight;
    return [x, y, z];
};
/**
 * Gets tiles needed to cover a bbox
 *
 * @param bbox [lngMin, latMin, lntMax, latMax]
 * @param z zoom
 * @returns tiles [[x, y, z], ...]
 */
var bboxToTiles = function (bbox, z) {
    var tlTile = pointToTileFraction(bbox[0], bbox[3], z);
    var brTile = pointToTileFraction(bbox[2], bbox[1], z);
    var tiles = [];
    // Floor so that we can apply gt and lt operator
    for (var x = Math.floor(tlTile[0]); x < brTile[0]; x++) {
        for (var y = Math.floor(tlTile[1]); y < brTile[1]; y++) {
            tiles.push([x, y, z]);
        }
    }
    return tiles;
};
/**
 * Get the bbox of a tile
 *
 * @param tile [x, y, z]
 * @returns [lngMin, latMin, lngMax, latMax]
 */
var tileToBBox = function (_a) {
    var x = _a[0], y = _a[1], z = _a[2];
    // Calculate total tiles at the zoom level
    var _b = getExtent(z), matrixWidth = _b[0], matrixHeight = _b[1];
    // Width and height in degrees
    var tileWidth = 360 / matrixWidth;
    var tileHeight = 180 / matrixHeight;
    // Normalized x and y
    // dx and dy correspond to the top left corner (xmin, ymax)
    var dx = x / matrixWidth;
    var dy = y / matrixHeight;
    // Convert to lng and lat
    var lng = dx * 360 - 180; // xMin (left)
    var lat = -dy * 180 + 90; // yMax (top)
    return [lng, lat - tileHeight, lng + tileWidth, lat];
};

// License MIT
// See https://github.com/datalyze-solutions/globalmaptiles/blob/master/globalmaptiles.js
// Modified API and syntax
// Degrees to radians
var D2R = Math.PI / 180;
// Radians to degrees
var R2D = 180 / Math.PI;
// Equitorial radius in meters (WGS 84)
var EARTH_RADIUS = 6378137;
// Equitorial circumferance in meters (WGS 84)
var EARTH_CIRCUMFERENCE = Math.PI * 2 * EARTH_RADIUS;
// Translate origin to corner
var ORIGIN_SHIFT = Math.PI * EARTH_RADIUS;
/**
 * Resolution (meters/pixel) for given zoom level (measured at Equator)
 * @param zoom
 * @returns
 */
var getResolution = function (zoom, tileSize) {
    return (EARTH_CIRCUMFERENCE / tileSize) / (1 << zoom); // Math.pow(2, zoom);
};
/**
 * Converts XY point from Spherical Mercator EPSG:900913 to lat/lon in WGS84 Datum
 *
 * @param xy
 * @returns
 */
var metersToLngLat = function (_a) {
    var mx = _a[0], my = _a[1];
    var lng = mx / ORIGIN_SHIFT * 180;
    var lat = my / ORIGIN_SHIFT * 180;
    lat = R2D * (2 * Math.atan(Math.exp(lat * D2R)) - (Math.PI / 2));
    return [lng, lat];
};
/**
 * Converts EPSG:900913 to pyramid pixel coordinates in given zoom level
 * @param xy
 * @param zoom
 * @returns
 */
var metersToPixels = function (_a, zoom, tileSize) {
    var mx = _a[0], my = _a[1];
    var res = getResolution(zoom, tileSize);
    var px = (mx + ORIGIN_SHIFT) / res;
    var py = (my + ORIGIN_SHIFT) / res;
    return [px, py];
};
/**
 * Converts pixel coordinates in given zoom level of pyramid to EPSG:900913
 * @param xy
 * @param zoom
 * @returns
 */
var pixelsToMeters = function (_a, zoom, tileSize) {
    var px = _a[0], py = _a[1];
    var res = getResolution(zoom, tileSize);
    var mx = px * res - ORIGIN_SHIFT;
    var my = py * res - ORIGIN_SHIFT;
    return [mx, my];
};
/**
 * Move the origin of pixel coordinates from bottom-left to top-left corner
 * @param xy
 * @param zoom
 * @returns
 */
var pixelsToScreenPixels = function (_a, zoom, tileSize) {
    var px = _a[0], py = _a[1];
    return [
        px,
        (tileSize << zoom) - py // transform y origin from bottom to top
    ];
};
/**
 * Move the origin of pixel coordinates from top-left to bottom-left corner
 * @param xy
 * @param zoom
 * @returns
 */
var screenPixelsToPixels = pixelsToScreenPixels;

var destinationToPixel = function (xy, zoom, tileSize) {
    var p = metersToPixels(xy, zoom, tileSize);
    return pixelsToScreenPixels(p, zoom, tileSize);
};
var pixelToDestination = function (xy, zoom, tileSize) {
    var p = screenPixelsToPixels(xy, zoom, tileSize);
    return pixelsToMeters(p, zoom, tileSize);
};
var destinationToSource = function (_a) {
    var x = _a[0], y = _a[1];
    return metersToLngLat([x, y]);
};
var sourceToPixel = function (lngLat, zoom, tileSize) {
    var extent = getExtent(zoom);
    return [
        normalizeLng(lngLat[0]) * extent[0] * tileSize,
        normalizeLat(lngLat[1]) * extent[1] * tileSize,
    ];
};
var destinationTileToSourceTiles = function (destinationRequest, options) {
    var _a;
    var lngLatBbox = __spreadArray(__spreadArray([], metersToLngLat([destinationRequest.bbox[0], destinationRequest.bbox[1]]), true), metersToLngLat([destinationRequest.bbox[2], destinationRequest.bbox[3]]), true);
    var zoom = destinationRequest.tile[2];
    var sourceTiles = bboxToTiles(lngLatBbox, (_a = zoom + options.zoomOffset) !== null && _a !== void 0 ? _a : 0);
    return sourceTiles.map(function (tile) {
        var bbox = tileToBBox(tile);
        return { tile: tile, bbox: bbox };
    });
};
var defaultOptions = {
    zoomOffset: -1
};
var epsg4326ToEpsg3857Presets = function (options) {
    var _a;
    var _options = {
        zoomOffset: (_a = options === null || options === void 0 ? void 0 : options.zoomOffset) !== null && _a !== void 0 ? _a : defaultOptions.zoomOffset,
    };
    return {
        destinationToPixel: destinationToPixel,
        pixelToDestination: pixelToDestination,
        destinationToSource: destinationToSource,
        sourceToPixel: sourceToPixel,
        destinationTileToSourceTiles: function (props) { return destinationTileToSourceTiles(props, _options); },
    };
};

exports.epsg4326ToEpsg3857Presets = epsg4326ToEpsg3857Presets;
exports.maplibreTileAdapterProtocol = maplibreTileAdapterProtocol;
//# sourceMappingURL=bundle.cjs.map
