/**
 * Resolution (meters/pixel) for given zoom level (measured at Equator)
 * @param zoom
 * @returns
 */
export declare const getResolution: (zoom: number, tileSize: number) => number;
/**
 * Converts given lat/lon in WGS84 Datum to XY in Spherical Mercator EPSG:900913
 *
 * @param lngLat
 * @returns
 */
export declare const lngLatToMeters: ([lng, lat]: number[]) => number[];
/**
 * Converts XY point from Spherical Mercator EPSG:900913 to lat/lon in WGS84 Datum
 *
 * @param xy
 * @returns
 */
export declare const metersToLngLat: ([mx, my]: number[]) => number[];
/**
 * Converts EPSG:900913 to pyramid pixel coordinates in given zoom level
 * @param xy
 * @param zoom
 * @returns
 */
export declare const metersToPixels: ([mx, my]: number[], zoom: number, tileSize: number) => number[];
/**
 * Converts pixel coordinates in given zoom level of pyramid to EPSG:900913
 * @param xy
 * @param zoom
 * @returns
 */
export declare const pixelsToMeters: ([px, py]: number[], zoom: number, tileSize: number) => number[];
/**
 * Returns a tile covering region in given pixel coordinates
 * @param xy
 * @returns
 */
export declare const pixelsToTile: ([px, py]: number[], zoom: number, tileSize: number) => number[];
/**
 * Move the origin of pixel coordinates from bottom-left to top-left corner
 * @param xy
 * @param zoom
 * @returns
 */
export declare const pixelsToScreenPixels: ([px, py]: number[], zoom: number, tileSize: number) => number[];
/**
 * Move the origin of pixel coordinates from top-left to bottom-left corner
 * @param xy
 * @param zoom
 * @returns
 */
export declare const screenPixelsToPixels: ([px, py]: number[], zoom: number, tileSize: number) => number[];
/**
 *
 * @param lngLat
 * @param zoom
 * @param tileSize
 * @returns tile [x, y, z]
 */
export declare const lngLatToTile: (lngLat: number[], zoom: number, tileSize: number) => number[];
/**
 *
 * @param xy
 * @param zoom
 * @param tileSize
 * @returns tile [x, y, z]
 */
export declare const metersToTile: (xy: number[], zoom: number, tileSize: number) => number[];
/**
 * Returns bounds of the given tile in EPSG:900913 coordinates
 * @param tile [x, y, z]
 * @param tileSize
 * @returns bbox [xmin, ymin, xmax, ymax]
 */
export declare const tileBbox: ([x, y, z]: number[], tileSize: number) => number[];
/**
 * Converts TMS tile coordinates to XYZ coordinates
 * Tile coord origin: XYZ is top-left, TMS is bottom-left
 * @param tile [x, y, z]
 * @returns tile [x, y, z]
 */
export declare const tmsToXyz: ([x, y, z]: number[]) => number[];
/**
 * Converts XYZ tile coordinates to TMS coordinates. Alias for tmsToXyz (same euqation).
 * Tile coord origin: XYZ is top-left, TMS is bottom-left
 * @param tile [x, y, z]
 * @returns tile [x, y, z]
 */
export declare const xyzToTms: ([x, y, z]: number[]) => number[];
/**
 * Converts TMS tile to microsoft quadkey
 * @param tile [x, y, z]
 * @returns
 */
export declare const tileToQuadkey: ([x, y, z]: number[]) => string;
/**
 * Converts microsoft quadkey to TMS tile
 * @param quadKey
 * @returns tile [x, y, z]
 */
export declare const quadkeyToTile: (quadkey: string) => number[];
